@@ -19,8 +19,10 @@ import (
 	"bytes"
 	"fmt"
 	"io"
+	"log"
 	"os"
 	"os/exec"
+	"strconv"
 	"strings"
 )
 
@@ -63,6 +65,10 @@ type Config struct {
 	// See https://github.com/sigstore/sigstore/blob/c645ceb9d075499f3a4b3f183d3a6864640fa956/pkg/oauthflow/flow.go#L49-L53
 	// for more details.
 	ConnectorID string
+	// TokenProviders select a OIDC token provider to use to fetch tokens. If not set, all providers are attempted.
+	// See https://github.com/sigstore/cosign/tree/main/pkg/providers for more details.
+	// Valid values are: [interactive, spiffe, google-workload-identity, google-impersonation, github-actions, filesystem, buildkite-agent]
+	TokenProvider string
 
 	// Timestamp Authority address to use to get a trusted timestamp
 	TimestampURL string
@@ -76,6 +82,11 @@ type Config struct {
 	CommitterName  string
 	CommitterEmail string
 	MatchCommitter bool
+
+	// Autoclose specifies whether to close window after successful authentication
+	Autoclose bool
+	// AutocloseTimeout specifies the time to wait before closing the window
+	AutocloseTimeout int
 }
 
 // Get fetches the gitsign config options for the repo in the current working
@@ -94,7 +105,9 @@ func Get() (*Config, error) {
 		ClientID: "sigstore",
 		Issuer:   "https://oauth2.sigstore.dev/auth",
 		// TODO: default to offline
-		RekorMode: "online",
+		RekorMode:        "online",
+		Autoclose:        true,
+		AutocloseTimeout: 6,
 	}
 
 	// Get values from config file.
@@ -117,8 +130,11 @@ func Get() (*Config, error) {
 		out.RedirectURL = envOrValue(fmt.Sprintf("%s_OIDC_REDIRECT_URL", prefix), out.RedirectURL)
 		out.Issuer = envOrValue(fmt.Sprintf("%s_OIDC_ISSUER", prefix), out.Issuer)
 		out.ConnectorID = envOrValue(fmt.Sprintf("%s_CONNECTOR_ID", prefix), out.ConnectorID)
+		out.TokenProvider = envOrValue(fmt.Sprintf("%s_TOKEN_PROVIDER", prefix), out.TokenProvider)
 		out.TimestampURL = envOrValue(fmt.Sprintf("%s_TIMESTAMP_SERVER_URL", prefix), out.TimestampURL)
 		out.TimestampCert = envOrValue(fmt.Sprintf("%s_TIMESTAMP_CERT_CHAIN", prefix), out.TimestampCert)
+		out.Autoclose = envOrValue(fmt.Sprintf("%s_AUTOCLOSE", prefix), fmt.Sprintf("%t", out.Autoclose)) == "true"
+		out.AutocloseTimeout, _ = strconv.Atoi(envOrValue(fmt.Sprintf("%s_AUTOCLOSE_TIMEOUT", prefix), fmt.Sprintf("%d", out.AutocloseTimeout)))
 	}
 
 	out.LogPath = envOrValue("GITSIGN_LOG", out.LogPath)
@@ -190,12 +206,23 @@ func applyGitOptions(out *Config, cfg map[string]string) {
 			out.LogPath = v
 		case strings.EqualFold(k, "gitsign.connectorID"):
 			out.ConnectorID = v
+		case strings.EqualFold(k, "gitsign.tokenProvider"):
+			out.TokenProvider = v
 		case strings.EqualFold(k, "gitsign.timestampServerURL"):
 			out.TimestampURL = v
 		case strings.EqualFold(k, "gitsign.timestampCertChain"):
 			out.TimestampCert = v
 		case strings.EqualFold(k, "gitsign.matchCommitter"):
 			out.MatchCommitter = strings.EqualFold(v, "true")
+		case strings.EqualFold(k, "gitsign.autoclose"):
+			out.Autoclose = strings.EqualFold(v, "true")
+		case strings.EqualFold(k, "gitsign.autocloseTimeout"):
+			if i, err := strconv.Atoi(v); err == nil && i > 0 {
+				out.AutocloseTimeout = i
+			} else {
+				log.Printf("invalid gitsign.autocloseTimeout value %q, defaulting to 6", v)
+				out.AutocloseTimeout = 6
+			}
 		}
 	}
 }
