@@ -34,6 +34,7 @@ import (
 	"github.com/sigstore/gitsign/internal/config"
 	"github.com/sigstore/gitsign/internal/fulcio/fulcioroots"
 	"github.com/sigstore/gitsign/internal/signerverifier"
+	"github.com/sigstore/gitsign/pkg/fulcio"
 	"github.com/sigstore/sigstore/pkg/oauth"
 	"github.com/sigstore/sigstore/pkg/oauthflow"
 	"github.com/sigstore/sigstore/pkg/signature"
@@ -193,8 +194,19 @@ func NewIdentityFactory(in io.Reader, out io.Writer) *IdentityFactory {
 
 func (f *IdentityFactory) NewIdentity(ctx context.Context, cfg *config.Config) (*Identity, error) {
 	clientID := cfg.ClientID
+
+	// Autoclose only works if we don't go through the identity selection page
+	// (otherwise it'll show a countdown timer that doesn't work)
+	if cfg.ConnectorID == "" {
+		cfg.Autoclose = false
+	}
+	html, err := oauth.GetInteractiveSuccessHTML(cfg.Autoclose, cfg.AutocloseTimeout)
+	if err != nil {
+		fmt.Println("error getting interactive success html, using static default", err)
+		html = oauth.InteractiveSuccessHTML
+	}
 	defaultFlow := &oauthflow.InteractiveIDTokenGetter{
-		HTMLPage: oauth.InteractiveSuccessHTML,
+		HTMLPage: html,
 		Input:    f.in,
 		Output:   f.out,
 	}
@@ -203,10 +215,27 @@ func (f *IdentityFactory) NewIdentity(ctx context.Context, cfg *config.Config) (
 	}
 	var authFlow oauthflow.TokenGetter = defaultFlow
 
-	if providers.Enabled(ctx) {
-		idToken, err := providers.Provide(ctx, clientID)
+	// If enabled, try OIDC token providers to get a token. unless the token provider is "interactive" (in which case always do default interactive flow).
+	var provider providers.Interface
+	if cfg.TokenProvider == "" && providers.Enabled(ctx) {
+		// If no token provider is set, look for any available provider to use.
+		provider = defaultFlowProvider{}
+	} else if cfg.TokenProvider != "" && cfg.TokenProvider != "interactive" {
+		fmt.Fprintln(f.out, "using token provider", cfg.TokenProvider)
+
+		// If a token provider is explicitly set always use it, unless it's "interactive",
+		// which means always use the default interactive flow.
+		p, err := providers.ProvideFrom(ctx, cfg.TokenProvider)
+		if err != nil {
+			return nil, fmt.Errorf("error getting token provider %q: %w", cfg.TokenProvider, err)
+		}
+		provider = p
+	}
+	if provider != nil {
+		idToken, err := provider.Provide(ctx, clientID)
 		if err != nil {
 			fmt.Fprintln(f.out, "error getting id token:", err)
+			return nil, fmt.Errorf("error getting id token: %w", err)
 		}
 		authFlow = &oauthflow.StaticTokenGetter{RawToken: idToken}
 	}
@@ -216,8 +245,8 @@ func (f *IdentityFactory) NewIdentity(ctx context.Context, cfg *config.Config) (
 		return nil, fmt.Errorf("generating private key: %w", err)
 	}
 
-	client, err := NewClient(cfg.Fulcio,
-		OIDCOptions{
+	client, err := fulcio.NewClient(cfg.Fulcio,
+		fulcio.OIDCOptions{
 			Issuer:      cfg.Issuer,
 			ClientID:    clientID,
 			RedirectURL: cfg.RedirectURL,
@@ -239,3 +268,13 @@ func (f *IdentityFactory) NewIdentity(ctx context.Context, cfg *config.Config) (
 		ChainPEM:   cert.ChainPEM,
 	}, nil
 }
+
+type defaultFlowProvider struct{}
+
+func (defaultFlowProvider) Enabled(ctx context.Context) bool {
+	return providers.Enabled(ctx)
+}
+
+func (defaultFlowProvider) Provide(ctx context.Context, audience string) (string, error) {
+	return providers.Provide(ctx, audience)
+}
